
comment !
From: "Hans_A.Lonnemo" <17362HAL@msu.edu>

You're welcome.  I have been coding for a very long time and have found it
immensely helpful to stick to certain coding standards (which I tend to use in
whatever language I am coding in).  One of these standards is to number my
subroutines and provide a comment both at the call and at the occurrence of a
routine.  The number makes it a lot easier to find the routine whether I am
editting online or looking at hardcopy and the comment usually makes it
unnecessary to go see what the routine is doing.  I really hate looking all
over for a routine or a label when I am studying somebody else's code.
However, when a routine is clearly 'canned' and of a utilitarian nature rather
than an integral part of the program (as PrintASCIIZ), I will give the routine
an understandable name instead.  This differentiation is carried over from my
BASIC programming where GOSUBs are numbered and CALLs are named.  A nice side
effect from this naming methodology is having all the local labels in a
routine named as well.  So when I said 300 series, I meant routines 301, 302,
303? (I can't recall how many there were right now) which are conveniently
labeled _30100 etc.  Another practice I highly recommend, but haven't seen
anybody else do, is to indent your code in control structures like you do in
high level languages and to liberally use vertical spacing to group statements
into functional clumps.  This makes the logic way easier to follow.  I have
often told people that you should be able to hang a listing on the wall and be
able to go to the other side of the room and point out what is what.  If you
can do this, it is certainly going to be much easier to keep your orientation
when you are rapidly paging through the code online.  (Oh, and the line
numbers help with this too)  Let me know what you do to increase the
performance of the code.

From: 17362HAL@MSU.EDU (H. Anders Lonnemo)
In article <BlchkS9cp-aH069yn@teleport.com>, kowens@teleport.com (Jeff Owens) says:

This is an example of a Heapsort.  It is embedded in a routine called LSORT.
LSORT is a replacement for the DOS SORT filter able to handle larger files
 than 64K.  It will only work in real mode.  You will find the sort routine in
 the 300 series of subroutines.  This code was written to work properly,
not necessarily for performance optimality, therefore you may be able to 
improve on it substantially.  I don't think there are any bugs, but if anybody
finds one, I'd be much obliged if you were to point it out.

Enjoy,
Anders

!

;==============================================================================
ASSUME CS:CODE, DS:DATA, SS:STACK
DATA SEGMENT
;==============================================================================

Rack            dw 30000 dup (?)
TopOfRack       dw 0

ErrorCount      db 0

Message1        db "Error during Read", 13, 10, 0
Message2        db "Error during Write", 13, 10, 0
Message3        db "File too big", 13, 10, 0
Message4        db "Too many lines", 13, 10, 0

TopOfSpace      dw 0, 0
TopOfData       dw 0, 0

;==============================================================================
DATA ENDS
CODE SEGMENT
PROGRAM:
;==============================================================================

        cld

        mov ax, ds:[2]

        mov dx, DATA
        mov ds, dx
        mov es, dx

        mov dx, 16
        mul dx

        mov TopOfSpace, ax
        mov TopOfSpace[2], dx

;---- Main Process

        call _10100 ; Read in the Entire File
        cmp ErrorCount, 0
        ja _9010

        call _20100 ; Build the Reference Rack
        cmp ErrorCount, 0
        ja _9010

        call _30100 ; Sort the Entire File
        cmp ErrorCount, 0
        ja _9010

        call _40100 ; Write out the Entire File
        cmp ErrorCount, 0
        ja _9010

;---- DOS Exit

_9010:  mov al, ErrorCount
        mov ah, 4Ch
        int 21h

;==============================================================================
_10100: ; Read in the Entire File

        push ds

        mov ax, BOTTOM
        mov ds, ax
        xor si, si

_10110: call DSSI_to_DLAX
        cmp dx, es:TopOfSpace[2]
        ja _10170
           jb _10120
              cmp ax, es:TopOfSpace
              jae _10140

_10120: mov dx, si
        mov cx, 8000h
        mov bx, 0
        mov ah, 3Fh
        int 21h
        jnc _10130

           mov ax, DATA
           mov ds, ax
           mov dx, offset DATA:Message1
           call PrintASCIIZ
           inc ErrorCount
           jmp short _10190

_10130: cmp ax, 0
        je _10140
           mov cx, ax
           call DSSI_to_DLAX
           add ax, cx
           adc dx, 0
           call DLAX_to_DSSI
           jmp short _10110

_10140: call DSSI_to_DLAX
        mov es:TopOfData, ax
        mov es:TopOfData[2], dx
        jmp short _10190

_10170:    mov ax, DATA
           mov ds, ax
           mov dx, offset DATA:Message3
           call PrintASCIIZ
           inc ErrorCount

_10190: pop ds

        ret

;==============================================================================
_20100: ; Build the Reference Rack

        push ds

        mov di, offset DATA:Rack

        mov ax, BOTTOM
        mov ds, ax
        xor si, si

_20110: call DSSI_to_DLAX
        cmp dx, es:TopOfData[2]
        ja _20140
           jb _20120
              cmp ax, es:TopOfData
              jae _20140

_20120:    cmp di, offset DATA:TopOfRack
           jb _20130
              mov ax, DATA
              mov ds, ax
              mov dx, offset DATA:Message4
              call PrintASCIIZ
              inc ErrorCount
              jmp short _20150

_20130:    stosw
           call DLAX_to_DSSI
           call _20200 ; Scan for End of Line
           mov ax, dx
           stosw
           jmp short _20110

_20140: mov es:TopOfRack, di

_20150: pop ds
        ret

;------------------------------------------------------------------------------
_20200: ; Scan for End of Line

        mov cx, si

_20210: lodsb
        cmp al, 13
        je _20240

        cmp al, 0
        je _20220

        cmp al, 26
        jne _20210

_20220:   mov byte ptr [si-1], 13
          mov byte ptr [si], 10

_20240: sub cx, si
        neg cx
        inc cx
        inc si

        cmp cx, 00FFh
        jae _20250
           mov dh, cl
         jmp short _20260
_20250:    mov dh, 0FFh
_20260:

        ret

;==============================================================================
_30100: ; Sort the Entire File

;---- First Pass

        mov bx, offset DATA:Rack
        add bx, 4

_30110: cmp bx, TopOfRack
        jae _30120
            push bx

            call ConvertOffsetToIndex
            call _30200 ; First While

            pop bx
            add bx, 4
            jmp short _30110


;---- Second Pass

_30120: mov bx, TopOfRack
        sub bx, 4

_30130: cmp bx, offset DATA:Rack
        jbe _30140
            push bx

            call ConvertOffsetToIndex
            mov cx, ax
            dec cx

            mov dx, 1
            call _31200 ; Swap 1st (AX) and 2nd (DX)
            call _30300 ; Second While

            pop bx
            sub bx, 4
            jmp short _30130

_30140: ret

;------------------------------------------------------------------------------
_30200: ; First While

_30210: cmp ax, 1
        jbe _30290
           mov dx, ax
           shr dx, 1
           call _31100 ; Compare 1st (AX) to 2nd (DX)
           jb _30290
           call _31200 ; Swap 1st (AX) and 2nd (DX)
           mov ax, dx
           jmp short _30210

_30290: ret

;------------------------------------------------------------------------------
_30300: ; Second While

        mov dx, 1
                
_30310: mov ax, dx
        add dx, dx

        cmp dx, cx
        ja _30390
           je _30330

              push ax
              mov ax, dx
              inc dx
              call _31100 ; Compare 1st (AX) to 2nd (DX)
              jb _30320
                 dec dx
_30320:       pop ax

_30330:    call _31100 ; Compare 1st (AX) to 2nd (DX)
           ja _30390
  
           call _31200 ; Swap 1st (AX) and 2nd (DX)
           jmp short _30310

_30390: ret

;------------------------------------------------------------------------------
_31100: ; Compare 1st (AX) to 2nd (DX)

        push ax
        push dx

        push ds
        push es

        call ConvertIndexToOffset
        mov si, bx

        mov ax, dx
        call ConvertIndexToOffset
        mov di, bx

        call DSSI_From_RackSI
        call ESDI_From_RackDI

_31110:    lodsb
           cmp al, 13
           je _31130

              mov ah, es:[di]
              inc di
              cmp ah, 13
              je _31120

                 cmp al, ah
                 je _31110

        jmp short _31190

_31120:       cmp ah, 12
              jmp short _31190

_31130:    cmp al, 14

_31190: pop es
        pop ds

        pop dx
        pop ax

        ret

;------------------------------------------------------------------------------
_31200: ; Swap 1st (AX) and 2nd (DX)

        push ax
        push dx

        call ConvertIndexToOffset
        mov si, bx

        mov ax, dx
        call ConvertIndexToOffset
        mov di, bx

        mov ax, [si]
        xchg ax, [di]
        mov [si], ax

        mov ax, [si+2]
        xchg ax, [di+2]
        mov [si+2], ax

        pop dx
        pop ax

        ret

;==============================================================================
_40100: ; Write out the Entire File

        push ds
        mov di, offset DATA:Rack

        mov ax, BOTTOM
        mov ds, ax
        xor si, si

_40110: cmp di, es:TopOfRack
        jae _40120
           call _40200 ; Write the Current Line to Output
           jmp short _40110

_40120:
        pop ds
        ret

;------------------------------------------------------------------------------
_40200: ; Write the Current Line to Output

        mov ax, es:[di]
        inc di
        inc di

        mov dx, es:[di]
        inc di
        inc di

        mov cl, dh
        xor ch, ch

        call DLAX_to_DSSI

        cmp cl, 0FFh
        jb _40210
           call _40300 ; Find the Size of the Line

_40210: mov dx, si
        mov bx, 1
        mov ah, 40h
        int 21h
        jnc _40220

           mov dx, offset DATA:Message2
           call PrintASCIIZ
           inc ErrorCount

_40220: ret

;------------------------------------------------------------------------------
_40300: ; Find the Size of the Line

        mov cx, si
        add si, 254

_40310: inc si
        cmp byte ptr [si], 10
        jne _40310

        xchg cx, si
        sub cx, si
        inc cx
        ret

;==============================================================================
PrintASCIIZ:
; On Entry
;  DS:DX Points to ASCIIZ String

        push ax
        push bx
        push cx
        push dx

        mov bx, dx

PA_010: cmp byte ptr [bx], 0
        je PA_020
           inc bx
           jmp short PA_010

PA_020: mov cx, bx
        sub cx, dx
        mov bx, 2
        mov ah, 40h
        int 21h

        pop dx
        pop cx
        pop bx
        pop ax

        ret

;==============================================================================
DSSI_to_DLAX:

        mov ax, ds
        mov dx, 16

        mul dx

        add ax, si
        adc dx, 0

        ret

;==============================================================================
DLAX_to_DSSI:

        push ax
        push dx

        xor dh, dh
        mov si, 16

        div si

        mov ds, ax
        mov si, dx

        pop dx
        pop ax

        ret

;==============================================================================
ConvertOffsetToIndex:

        mov ax, bx
        sub ax, offset DATA:Rack
        shr ax, 1
        shr ax, 1
        inc ax
        ret

;==============================================================================
ConvertIndexToOffset:

        mov bx, ax
        dec bx
        shl bx, 1
        shl bx, 1
        add bx, offset DATA:Rack
        ret

;==============================================================================
DSSI_From_RackSI:

        push ax
        push dx

        mov ax, [si]
        mov dx, [si+2]

        xor dh, dh
        mov si, 16

        div si

        mov ds, ax
        mov si, dx

        pop dx
        pop ax

        ret

;==============================================================================
ESDI_From_RackDI:

        push ax
        push dx

        mov ax, es:[di]
        mov dx, es:[di+2]

        xor dh, dh
        mov di, 16

        div di

        mov es, ax
        mov di, dx

        pop dx
        pop ax

        ret

;=================================================================CODE ENDS
STACK SEGMENT STACK
        dw 1000 dup (?)
STACK ENDS
BOTTOM SEGMENT
BOTTOM ENDS
END PROGRAM

